################################################################################
# R code to prepare data, call winBUGS code to perform Bayesian wombling,
# and plot output.
# 
# 21 April 2010
# M. C. Fitzpatrick
# mfitzpatrick@umces.edu
# 
# Most of this code is based on hard work by Brad Carlin & his students/post-
# docs. I have simply assembled many pieces into one place.
# In some instances, files available from:
# http://www.biostat.umn.edu/~brad/software.html are needed. See comments below.
# 
# The anlyses require a shapefile with a response of interest (in this case
# month of first infestation) and corresponding covariates
################################################################################


###     chunk 1 - set wd and load libraries       ##############################
setwd("...")
source("womblingFuncs.R")
library(R2WinBUGS)
library(maptools)
library(spdep)
library(coda)
library(RColorBrewer)
library(classInt)     
library(sp)
################################################################################


###     chunk 2 - build adjacency & edge info    ###############################
#  Given a shapefile, this R code creates:
#    (1) an areal adjacency matrix using maptools,  
#    (2) an edge adjacency matrix (indicating which edges touch each other)

# based on code provided from B. Carlin's website:
# http://www.biostat.umn.edu/~brad/software/getEdges_code.txt

# Also will need following two .exe files from B. Carlin's website
# (1) matchzip.exe
# (2) edgeneig.exe
# downloaded from: http://www.biostat.umn.edu/~brad/software/tutorial.zip

setwd(".../edgefolder")
# output files will be saved under current directory
# copy "matchzip.exe" and "edgeneig.exe" to directory "edgefolder"  

map <- readShapeSpatial(".../hwa_wombling.shp")

# Use maptools to get polygon adjacency matrix
# two areas are neighbors if they share common edges with length > 0
nb.r <- poly2nb(map, queen=F)    
mat <- nb2mat(nb.r, style="B") # mat is the 0/1 adjacency matrix
n.site <- dim(mat)[1]          # n.site: number of areas
n.edge <- sum(mat)/2           # n.edge: number of unique pairs


SEind1 <- SEind2 <- 0
matmy <- mat
for(i in 1:(n.site-1)){
    for(j in (i+1):n.site){
        if (mat[i,j]>0) {SEind1<-c(SEind1,i)
            SEind2<-c(SEind2,j)
            matmy[i,j]<-matmy[j,i]<-length(SEind1)-1
        }
    }
}

SEind1 <- SEind1[-1] # edges sorted by row of upper triangle of the adj. matrix
SEind2 <- SEind2[-1] # SEind1[k]=i and SEind2[k]=j => kth edge is edge ij

dput(SEind1,"SEind1.txt")
dput(SEind2,"SEind2.txt")
dput(mat, "W.txt")

# create adjacency information needed for WinBUGS
mkAdj <- function(W){
    n <- nrow(W)
    adj <- 0
    for(i in 1:n){
        for(j in 1:n){
            if(W[i,j]==1){adj<-append(adj,j)
            }
        }
    }
adj <- adj[-1]
return(adj)
}

dput(mkAdj(mat),"Sadj.txt")
dput(as.vector(rowSums(mat)),"Snum.txt")

# Create adj. matrix for the edges 

# 1. prepare needed files and save them #
#    under the directory where you have #
#    matchzip.exe and edgeneig.exe      #

# Dump out the coordinates (by polygon) #
# Default order for polygons is by first column of map@data #

for(i in 1:n.site){
    write(t(map@polygons[[i]]@Polygons[[1]]@coords), paste(i,".txt",sep=""),
          ncolumns=2)
}
    
# Dump out SEind, the site-edge correspondence table #
write(rbind(SEind1, SEind2), paste("SEind.txt",sep=""), ncolumns=2 )

# 2. Double click "matchzip.exe". A dos window will pop up. Type in SEind.txt. #
#  This will produce many (n.edge) files at the current directory.             #
#  Then use the following code to prepare the edge plotting code.              #
#  "edgelines" should be dumped out and called in later when make edge plots.  #

edgelines <- vector(mode="list",length=n.edge)

for(i in 1:n.edge){
    edgelines[[i]] <- read.table(paste("output-",i,".txt",sep=""), header=F,
                                        na.strings="*")
}

edges <- edgelines
dput(edgelines,"edgelines.txt")

# 3. Double click "edgeneig.exe". A dos window will pop up. Type in SEind.txt  #
#  Two files will be produced (may take a while):                              #
#  file 1 is the upper triangular of the W matrix for the edges;               #
#  file 2 is the number of 1s for each row of the upper triangular matrix.     #

#  4. Produce the Wstar matrix which provides neighborhoods of the edges       #

tempn <- scan(paste("file2.txt",sep=""))
tempneig <- scan(paste("file1.txt",sep=""))

Wstar <- matrix(0, nrow=n.edge, ncol=n.edge)
start <- 1
end <- 0
    
for(i in 1:n.edge){
    if (tempn[i]>0){ 
        start <- end+1
        end <- start + tempn[i]-1
        neig <- tempneig[start:end]
        l <- tempn[i]
        for (k in 1:l){
            Wstar[i,neig[k]]<-Wstar[neig[k],i]<-1
        }
    }
} 

edge.adj <- mkAdj(Wstar)
dput(edge.adj, "edge.adj.txt")
edgeSum <- as.vector(rowSums(Wstar))
dput(edgeSum, "edgeSum.txt")
dput(Wstar,"Wstar.txt")

# 5. Remove files not needed any more #
for(i in 1:n.edge){
    unlink(paste(i,".txt",sep=""))
    unlink(paste("output-",i,".txt",sep=""))
}

unlink(paste("file1.txt",sep=""))
unlink(paste("file2.txt",sep=""))
unlink(paste("SEind.txt",sep=""))
################################################################################


###     chunk 3 - Areal wombling model      ####################################
# Three global covariates, spatial error                             
# mu = beta0 + beta1*X1 + beta2*X2 + beta3*X3 + phi 

map <- readShapeSpatial(".../hwa_wombling.shp")

temp <- map@data$WINTERTEMP
pop <- log(map@data$POP2000)
hemlock <- log(map@data$HEMLOCK)

Y <- (map$YEARINFEST-1951)*12
X1 <- pop
X2 <- hemlock
X3 <- temp


n.areas = length(Y)
adj <- dget(".../edgefolder/Sadj.txt")
num = dget(".../edgefolder/Snum.txt")
sumNumNeigh = sum(num)

# indexes required for plotting
ind1 <- ind2 <- rep(0,length(num))
ind1[1] <- 1
for(i in 1:length(num)){j <- i+1; ind1[j] <- num[i] + ind1[i]}
ind1 <- ind1[1:length(num)]
for(i in 1:length(num)){j <- i-1; ind2[i] <- ind1[i] + num[i]-1}

params <- 5 # number of parameters in the model

# initial values
phi1 <- rep(-10,n.areas)
phi2 <- rep(0,n.areas)
phi3 <- rep(10,n.areas)

inits.mod5 <- list(list(phi=phi1, sdy=15, sdphi=5, beta=rep(-5, params)),
list(phi=phi2, sdy=25, sdphi=5, beta=rep(0, params)), list(phi=phi3, sdy=35,
sdphi=5, beta=rep(10, params)))

dat.in.mod5 <- list("Y", "n.areas", "num", "sumNumNeigh", "adj", "X1", "X2",
                    "X3", "X4")

# call to Winbugs
mod <- bugs(data=dat.in.mod5, inits.mod5,
            model.file=".../areal_wombling.bug",
            parameters.to.save=c("SLDRhat", "beta", "tau.err", "phi",
            "tau.phi"), n.chains = length(inits.mod5), n.iter=200000,
            n.burnin=100000, save.history=F, debug=TRUE,
            bugs.directory=".../WinBUGS14/", working.directory="...")

# read & summarize coda files
mod.coda <- read.coda.interactive()
# codaIndex.txt, coda1.txt, coda2.txt, coda3.txt
dimnames(mod.coda$coda1.txt)
samps <- mcmc.list(mcmc(mod.coda$coda1.txt[,c(323:328,651,652)]),
mcmc(mod.coda$coda2.txt[,c(323:328,651,652)]),
mcmc(mod.coda$coda3.txt[,c(323:328,651,652)]))
xyplot(samps)
gelman.plot(samps)
densityplot(samps)

merge.chains <- (mod.coda$coda1.txt + mod.coda$coda2.txt + mod.coda$coda3.txt)/3
SLDRhat <- merge.chains[,1:322]

rows <- nrow(SLDRhat)
cols <- ncol(SLDRhat)
phi <- merge.chains[,329:650]

# calculate posterior estimates of mu (sldrhat)
SLDRhat.samp <- matrix(0, ncol=cols, nrow=rows)
phi.samp <- matrix(0, ncol=cols, nrow=rows)
for(i in 1:n.areas){
    from<-(i-1)*rows+1
    to<-i*rows
    SLDRhat.samp[,i]<- SLDRhat[from:to]
    phi.samp[,i] <- phi[from:to]
}

#c alculate differences in spread dates across county edges
delta.sldr <- matrix(0, ncol=sum(num)/2,nrow=rows)
delta.phi <- matrix(0, ncol=sum(num)/2,nrow=rows)
k <- 0
for( i in 1:n.areas){
    for(j in ind1[i]:ind2[i]){
        if(adj[j]>i){
            k<-k+1
            delta.sldr[,k]<-abs(SLDRhat.samp[,i] - SLDRhat.samp[,adj[j]])
            delta.phi[,k]<-abs(phi.samp[,i] - phi.samp[,adj[j]])}}}

# Boundary likelihood values
p.sldr.5years <- apply(apply(delta.sldr,2,cut.func.5years)/rows,2,sum)
p.phi.5years <- apply(apply(delta.phi,2,cut.func.5years)/rows,2,sum)

# color palette for plotting boundaries
n.col = 4 
col.br <- colorRampPalette(c("gray", "lightpink2", "red2", "red4"))
col.pal <- col.br(n.col)

# breaks for boundary groupings & legend text
br <- c(0.0,0.4,0.6,0.9,1.0)
leg.txt <- paste("(",br[n.col]," ~ ",br[n.col+1],")",sep="")
for(i in (n.col-1):1){
    leg.txt <- append(leg.txt, paste("(", br[i], " ~ ", br[i+1], ")", sep=""),)
}
leg.txt <- rev(leg.txt)

# Plot maps with boundary probabilities
edgelines <- dget(".../edgefolder/edgelines.txt")

# map of mu-based booundaries
probPlot(map, edgelines, p.sldr.5years, n.col, add=F, col.pal=col.pal)
legend(locator(), legend=leg.txt, col=col.pal, lty="solid", lwd=c(2,3,4,5),
       cex=1.8, ncol=1,
bty="n", title="Boundary Probability")

# map of phi-based boundaries
probPlot(map, edgelines, p.phi.5years, n.col, add=F, col.pal=col.pal)
legend(locator(), legend=leg.txt, col=col.pal, lty="solid", lwd=c(2,3,4,5),
       cex=1.8, ncol=1,
bty="n", title="Boundary Probability")
################################################################################


###     chunk 4 - Local edge wombling model      ###############################
# Three global covariates, spatial error                             
# model will not run using R2WinBUGS for some reason
# must copy and paste model, inits, data and run directly in winBUGS


map <- readShapeSpatial(".../hwa_wombling.shp")
Y <- (map$YEARINFEST-1951)*12
mapDat <- map@data[,7:9]

# prepare data for edge wombling

# edge deltas
SEind1 <- dget(".../edgefolder/SEind1.txt")
SEind2 <- dget(".../edgefolder/SEind2.txt")
edgelines <- dget(".../edgefolder/edgelines.txt")
n.edge <- length(SEind1)

deltaY <- weight.calculate(Y, SEind1, SEind2, n.edge)

deltaCov <- matrix(NA, n.edge, ncol(mapDat))
for(i in 1:ncol(mapDat)){
    covar <- mapDat[,i]
    dx <- delta.calculate(covar, SEind1, SEind2, n.edge)
    deltaCov[,i] <- dx
}

colnames(deltaCov) <- names(mapDat)

Y <- deltaY

adj = dget("...edgefolder/edge.adj.txt")
num = dget("...edgefolder/edgeSum.txt")
n.areas = length(Y)
sumNumNeigh = sum(num)

X1 <- deltaCov[,"POP2000"]
X2 <- deltaCov[,"HEMLOCK"]
X3 <- deltaCov[,"WINTERTEMP"]

params <- 4 # number of parameters in the model

# initial values
phi1 <- rep(10, n.areas)
phi2 <- rep(0, n.areas)
phi3 <- rep(-1, n.areas)

phi1[which(num==0)] <- NA # for edges with no neighbors
phi2[which(num==0)] <- NA
phi3[which(num==0)] <- NA

psi1 <- rep(10, length(which(num==0))) # for edges with no neighbors
psi2 <- rep(1, length(which(num==0)))
psi3 <- rep(0, length(which(num==0)))

inits.mod10 <- list(list(psi=psi1, phi=phi1, sdy=15, sdphi=5, sdpsi=1,
                         beta=rep(5, params)), list(psi=psi2, phi=phi2, sdy=25,
                         sdphi=5, sdpsi=5, beta=rep(-2, params)), list(psi=psi3,
                         phi=phi3, sdy=35, sdphi=5, sdpsi=5, beta=rep(-50,
                         params)))
edit(inits.mod10)

dat.in.mod10 <- list(sumNumNeigh=sumNumNeigh, n.areas=n.areas, Y=Y,  num=num,
                     adj=adj, X1=X1, X2=X2, X3=X3)
edit(dat.in.mod10)


#call to Winbugs
mod10 <- bugs(data=dat.in.mod10, inits.mod10, model.file="...edge_womble.bug",
parameters.to.save=c("SLDRhat", "beta", "tau.err", "phi", "psi"),
n.chains = length(inits.mod10), n.iter=20000, n.burnin=10000,
save.history=F, debug=TRUE, bugs.directory=".../WinBUGS14/", codaPkg=T,
working.directory="...")

# read & summarize coda files
mod.coda <- read.coda.interactive()
# codaIndex.txt, coda1.txt, coda2.txt, coda3.txt
dimnames(mod.coda$coda1.txt)
samps <- mcmc.list(mcmc(mod.coda$coda1.txt[,c(794:798, 1592:1594)]),
mcmc(mod.coda$coda2.txt[,c(794:798, 1592:1594)]),
mcmc(mod.coda$coda3.txt[,c(794:798, 1592:1594)]))
xyplot(samps)
gelman.plot(samps)
densityplot(samps)

merge.chains <- (mod.coda$coda1.txt + mod.coda$coda2.txt + mod.coda$coda3.txt)/3
SLDRhat <- merge.chains[,1:793]
phi <- merge.chains[,799:1576]
psi <- merge.chains[,1577:1591]

rows <- nrow(SLDRhat)

# Boundary likelihood values at 5 years
p.sldr.5years <- apply(apply(abs(SLDRhat),2,cut.func.5years)/rows,2,sum)
p.phi.5years <- apply(apply(abs(phi),2,cut.func.5years)/rows,2,sum)
p.psi.5years <- apply(apply(abs(psi),2,cut.func.5years)/rows,2,sum)

num1 <- ifelse(num==0,0,1)

#combine island vector with index and sort
indx <- seq(1:length(Y))
srt <- as.data.frame(cbind(num1, indx))
srt <- srt[order(srt$num1, srt$indx),]

# bind phi and psi and then to srt df
phiX <- c(p.psi.5years, p.phi.5years)
srt <- cbind(srt,phiX)

# sort to original order and extract new psi vector
phi.df <- srt[order(srt$indx),]
p.phi.5years <- phi.df$phiX

# color palette for plotting boundaries
n.col = 4 
col.br <- colorRampPalette(c("gray", "lightpink2", "red2", "red4"))
col.pal <- col.br(n.col)

# breaks for boundary groupings & legend text
br <- c(0.0,0.4,0.6,0.9,1.0)
leg.txt <- paste("(",br[n.col]," ~ ",br[n.col+1],")",sep="")
for(i in (n.col-1):1){
    leg.txt <- append(leg.txt, paste("(", br[i], " ~ ", br[i+1], ")", sep=""),)
}
leg.txt <- rev(leg.txt)

# Plot maps with boundary probabilities
edgelines <- dget(".../edgefolder/edgelines.txt")

# map of mu-based booundaries
probPlot(map, edgelines, p.sldr.5years, n.col, add=F, col.pal=col.pal)
legend(locator(), legend=leg.txt, col=col.pal, lty="solid", lwd=c(2,3,4,5),
       cex=1.8, ncol=1, bty="n", title="Boundary Probability")

# map of phi-based boundaries
probPlot(map, edgelines, p.phi.5years, n.col, add=F, col.pal=col.pal)
legend(locator(), legend=leg.txt, col=col.pal, lty="solid", lwd=c(2,3,4,5),
       cex=1.8, ncol=1, bty="n", title="Boundary Probability")

# Boundary likelihood values at 3 years
p.sldr.3years <- apply(apply(abs(SLDRhat),2,cut.func.3years)/rows,2,sum)
p.phi.3years <- apply(apply(abs(phi),2,cut.func.3years)/rows,2,sum)
p.psi.3years <- apply(apply(abs(psi),2,cut.func.3years)/rows,2,sum)

num1 <- ifelse(num==0,0,1)

# combine island vector with index and sort
indx <- seq(1:length(Y))
srt <- as.data.frame(cbind(num1, indx))
srt <- srt[order(srt$num1, srt$indx),]

# bind phi and psi and then to srt df
phiX <- c(p.psi.3years, p.phi.3years)
srt <- cbind(srt,phiX)

# sort to original order and extract new psi vector
phi.df <- srt[order(srt$indx),]
p.phi.3years <- phi.df$phiX

# map of mu-based booundaries
probPlot(map, edgelines, p.sldr.3years, n.col, add=F, col.pal=col.pal)
legend(locator(), legend=leg.txt, col=col.pal, lty="solid", lwd=c(2,3,4,5),
       cex=1.8, ncol=1, bty="n", title="Boundary Probability")

# map of phi-based boundaries
probPlot(map, edgelines, p.phi.3years, n.col, add=F, col.pal=col.pal)
legend(locator(), legend=leg.txt, col=col.pal, lty="solid", lwd=c(2,3,4,5),
       cex=1.8, ncol=1, bty="n", title="Boundary Probability")
################################################################################